@@ -612,7 +612,7 @@ class ParametersLEDET:
         f_mag_X_all = BX / Inom
         f_mag_Y_all = BY / Inom
         f_mag = (f_mag_X_all**2 + f_mag_Y_all**2) ** 0.5
-        B = f_mag * self.Inputs.I00
+        B = f_mag *self.Inputs.I00
 
         B[B > 10E6]=10E-6
         return B
@@ -634,20 +634,29 @@ class ParametersLEDET:
         cable.A_CableInsulated = (self.Inputs.wBare_inGroup+2*self.Inputs.wIns_inGroup) \
                                * (self.Inputs.hBare_inGroup+2*self.Inputs.hIns_inGroup)
         B = self.acquireBField(ROXIE_File)
+
         if max(self.Inputs.nStrands_inGroup) > 1:
             strandCount = 0
             GroupCount = 0
+            nStrands_inGroup = self.Inputs.nStrands_inGroup
+            ds_inGroup = self.Inputs.ds_inGroup
+            if any(nStrands_inGroup % 2 == 1) and any(nStrands_inGroup != 1):
+                for g in range(len(self.Inputs.nT)):
+                    if (nStrands_inGroup[g] % 2 == 1) & (nStrands_inGroup[g] > 1):
+                        ds_inGroup[g] = ds_inGroup[g] * np.sqrt(nStrands_inGroup[g] / (nStrands_inGroup[g] - 1))
+                        nStrands_inGroup[g] = nStrands_inGroup[g] - 1
+
             Bcopy = np.zeros(int(sum(self.Inputs.nT)))
             for i in range(int(sum(self.Inputs.nT))):
-                Bcopy[i] = sum(B[int(strandCount):int(strandCount+self.Inputs.nStrands_inGroup[GroupCount])])/self.Inputs.nStrands_inGroup[int(GroupCount)]
+                Bcopy[i] = sum(B[int(strandCount):int(strandCount+nStrands_inGroup[GroupCount])])/nStrands_inGroup[int(GroupCount)]
                 TurnSum = sum(self.Inputs.nT[0:GroupCount+1])
                 if i>TurnSum: GroupCount = GroupCount + 1
-                strandCount = strandCount + self.Inputs.nStrands_inGroup[GroupCount]
+                strandCount = strandCount + nStrands_inGroup[GroupCount]
             B = Bcopy
             cable.f_SC = self.Inputs.f_SC_strand_inGroup * \
-                         (self.Inputs.nStrands_inGroup* (np.pi/4)*(self.Inputs.ds_inGroup**2)) / cable.A_CableInsulated
+                         (nStrands_inGroup* (np.pi/4)*(ds_inGroup**2)) / cable.A_CableInsulated
             cable.f_ST = (1 - self.Inputs.f_SC_strand_inGroup) * \
-                         (self.Inputs.nStrands_inGroup* (np.pi/4)*(self.Inputs.ds_inGroup**2)) / cable.A_CableInsulated
+                         (nStrands_inGroup* (np.pi/4)*(ds_inGroup**2)) / cable.A_CableInsulated
         else:
             cable.f_SC = self.Inputs.f_SC_strand_inGroup * \
                          (self.Inputs.wBare_inGroup * self.Inputs.hBare_inGroup) / cable.A_CableInsulated
