@@ -32,6 +32,7 @@ class ParametersCOSIM:
             self.nameMagnet = "_0"
         self.circuitName = nameCircuit
         self.nameMagnet = nameMagnet
+        self.executionOrder = []
 
         # Load and set the default config files (using ResourceReader allows reading from a "hidden" resource folder)
         self.nameTemplateConfigFileCosim = ResourceReader.getResourcePath(os.path.join('sing', 'STEAMConfig.json'))
@@ -65,20 +66,24 @@ class ParametersCOSIM:
                 os.mkdir(nameFolderLedetModel)
             if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET')):
                 os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET'))
-            if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET//'+ self.nameMagnet+"//")):
-                os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET//'+ self.nameMagnet+"//"))
-            if not os.path.isdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Input//"):
-                os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Input//")
-            if not os.path.isdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Input//Initialize variables//"):
-                os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Input//Initialize variables//")
-            if not os.path.isdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Input//Control current input//"):
-                os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Input//Control current input//")
-            if not os.path.isdir(nameFolderLedetModel + "//Field maps//"):
-                os.mkdir(nameFolderLedetModel + "//Field maps//")
-            if not os.path.isdir(nameFolderLedetModel + "//Field maps//" + self.nameMagnet):
-                os.mkdir(nameFolderLedetModel + "//Field maps//" + self.nameMagnet)
-            if not os.path.isdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Output//"):
-                os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet + "//Output//")
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet)):
+                os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Input")):
+                os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Input"))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel,"LEDET", self.nameMagnet, "Input","Initialize variables")):
+                os.mkdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Initialize variables"))
+                f = open(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Initialize variables", " .gitkeep"), "w")
+                f.close()
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Input","Control current input")):
+                os.mkdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Control current input"))
+                f = open(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Control current input", " .gitkeep"), "w")
+                f.close()
+            if not os.path.isdir(os.path.join(nameFolderLedetModel , "Field maps")):
+                os.mkdir(os.path.join(nameFolderLedetModel,"Field maps"))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "Field maps", self.nameMagnet)):
+                os.mkdir(os.path.join(nameFolderLedetModel, "Field maps", self.nameMagnet))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet ,"Output")):
+                os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Output"))
 
         if N_LEDET > 1:
             for i in range(1, N_LEDET+1):
@@ -88,22 +93,24 @@ class ParametersCOSIM:
                     os.mkdir(nameFolderLedetModel)
                 if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET')):
                     os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET'))
-                if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET//' + self.nameMagnet[i-1] + "//")):
-                    os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET//' + self.nameMagnet[i-1] + "//"))
-                if not os.path.isdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Input//"):
-                    os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Input//")
-                if not os.path.isdir(
-                        nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Input//Initialize variables//"):
-                    os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Input//Initialize variables//")
-                if not os.path.isdir(
-                        nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Input//Control current input//"):
-                    os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Input//Control current input//")
-                if not os.path.isdir(nameFolderLedetModel + "//Field maps//"):
-                    os.mkdir(nameFolderLedetModel + "//Field maps//")
-                if not os.path.isdir(nameFolderLedetModel + "//Field maps//" + self.nameMagnet[i-1]):
-                    os.mkdir(nameFolderLedetModel + "//Field maps//" + self.nameMagnet[i-1])
-                if not os.path.isdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Output//"):
-                    os.mkdir(nameFolderLedetModel + "//LEDET//" + self.nameMagnet[i-1] + "//Output//")
+                if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet[i-1])):
+                    os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet[i-1]))
+                if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[i-1], "Input")):
+                    os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[i-1], "Input"))
+                if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[i-1], "Input//Initialize variables")):
+                    os.mkdir( os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[i-1], "Input", "Initialize variables"))
+                    f = open(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[i-1], "Input", "Initialize variables", " .gitkeep"), "w")
+                    f.close()
+                if not os.path.isdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet[i-1],"Input","Control current input")):
+                    os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[i-1], "Input", "Control current input"))
+                    f = open(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[i-1], "Input", "Control current input", " .gitkeep"), "w")
+                    f.close()
+                if not os.path.isdir(os.path.join(nameFolderLedetModel,"Field maps")):
+                    os.mkdir(os.path.join(nameFolderLedetModel,"Field maps"))
+                if not os.path.isdir(os.path.join(nameFolderLedetModel,"Field maps",self.nameMagnet[i-1])):
+                    os.mkdir(os.path.join(nameFolderLedetModel,"Field maps",self.nameMagnet[i-1]))
+                if not os.path.isdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet[i-1],"Output")):
+                    os.mkdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet[i-1], "Output"))
 
     def copyConfigFiles(self, N_LEDET = 1):
         '''
@@ -227,6 +234,7 @@ class ParametersCOSIM:
         data["relTolerance"] = relTolerance
         data["absTolerance"] = absTolerance
         data["executionOrder"] = executionOrder
+        self.executionOrder = executionOrder
         data["executeCleanRun"] = executeCleanRun
         if N_LEDET >1:
             for i in range(2, N_LEDET+1):
@@ -280,7 +288,7 @@ class ParametersCOSIM:
                     if executionOrder.count(2) > 1:
                         idxLEDET = LEDETExecutable.find('LEDET')
                         data["solverPath"] = LEDETExecutable[0:idxLEDET + 5] + "_"+str(i) + LEDETExecutable[idxLEDET + 5:]
-                        print('LEDET parallel solving, performed from: ', data["solverPath"])
+                        print('COSIM Parallel mode: LEDET executable ',i,' should be: ', data["solverPath"],'.')
                     else: data["solverPath"] = LEDETExecutable
                     data["modelName"] = self.nameMagnet[i-1]
                     data["simulationNumber"] = str(SimulationNumber[i-1])
@@ -449,6 +457,8 @@ class ParametersCOSIM:
                 json.dump(struct_total[i], f, indent=4)
                 f.write('\n')
 
+        print("I/O done!")
+
     def FindLibPath(self, File):
         path = os.path.abspath(os.path.join(os.getcwd(), os.pardir))
         for i in range(5):
@@ -459,6 +469,28 @@ class ParametersCOSIM:
         print("Could not find PSPICE library. Please add manually.")
         return False
 
+    def CopyCircuit(self, FileOld, FileNew):
+        inOption = 0
+        inAutoConverge = 0
+
+        fN = open(FileNew, 'w+')
+        with open(FileOld) as fp:
+            for ls in fp:
+                # print(ls)
+                if ls.startswith("*.OPTION") or ls.startswith("* .OPTION"):
+                    fN.write(ls[1:])
+                    inOption = 1
+                elif ls.startswith("*.AUTOCONVERGE") or ls.startswith("* .AUTOCONVERGE"):
+                    fN.write(ls[1:])
+                    inAutoConverge = 1
+                elif ls.startswith("*+") or ls.startswith("* +"):
+                    if inOption or inAutoConverge: fN.write(ls[1:])
+                else:
+                    inOption = 0
+                    inAutoconverge = 0
+                    fN.write(ls)
+            fN.close()
+
     def copyCOSIMfiles(self, nameFileSING, StimulusFile, nameMagnet, N_LEDET=1, ManuallyStimulusFile = ''):
         LEDETFiles = os.path.abspath(os.path.join(os.path.abspath(os.path.join(os.getcwd(), os.pardir)),os.pardir))
         LEDETFiles = os.path.join(LEDETFiles, "steam-ledet-input")
@@ -473,8 +505,11 @@ class ParametersCOSIM:
         nameFolderPSpiceModel = os.path.join(nameFolderCosimModel, 'PSpice')
         if not os.path.isdir(nameFolderPSpiceModel):
             os.mkdir(nameFolderPSpiceModel)
-        makeCopyFile(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
+
+        self.CopyCircuit(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
+        # makeCopyFile(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
         makeCopyFile(StimulusFile, os.path.join(nameFolderPSpiceModel, 'ExternalStimulus.stl'))
+        print("Copy PSPICE done!")
 
         LEDET_Fs = []
         for i in range(N_LEDET):
@@ -507,20 +542,24 @@ class ParametersCOSIM:
 
     def prepareLEDETFiles(self, files, N_PAR=1):
         TurnNumber = 0
+        steps = 0
 
         if type(files) == list:
             for i in range(len(files)):
                 file = files[i]
-                Tnew = self.prepareSingleLEDETFile(file)
-                TurnNumber = TurnNumber + Tnew
+                [Tnew, Snew] = self.prepareSingleLEDETFile(file)
+                if self.executionOrder[i+1] == self.executionOrder[i]:
+                    TurnNumber = TurnNumber + Tnew
+                    steps = steps + Snew
             TurnNumber = int(TurnNumber / len(files))
+            steps = int(steps / len(files))
         else:
-            TurnNumber= self.prepareSingleLEDETFile(files)
+            [TurnNumber, steps] = self.prepareSingleLEDETFile(files)
 
         ## Do some kind of check
         TurnNumber_All = TurnNumber*N_PAR
 
-        print(TurnNumber_All,' Turns will be simulated in parallel.')
+        print(TurnNumber_All,' Turns are planned to be simulated in parallel with about ', steps, ' time steps.')
         if TurnNumber_All >= 10000:
             print("You are trying to simulate ",str(TurnNumber_All),"in parallel mode. \n")
             maxN_PAR = np.floor(10000/TurnNumber)
@@ -534,5 +573,11 @@ class ParametersCOSIM:
         a.Options.flag_saveMatFile = 0
         a.Options.flag_automaticRefinedTimeStepping = 0
         a.writeFileLEDET(file)
+
         TurnNumber = max(a.Inputs.HalfTurnToInductanceBlock)
-        return TurnNumber
+        tV = a.Options.time_vector_params
+        steps = 0
+        for i in range(0,len(tV),3):
+            steps = steps+ (abs(tV[i])-abs(tV[i+2]))/tV[i+1]
+
+        return [TurnNumber, steps]
